package com.hfkj.service.user.impl;

import com.hfkj.common.exception.BaseException;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.dao.BsUserAccountMapper;
import com.hfkj.entity.BsUserAccount;
import com.hfkj.entity.BsUserAccountExample;
import com.hfkj.entity.BsUserAccountRecord;
import com.hfkj.service.user.BsUserAccountRecordService;
import com.hfkj.service.user.BsUserAccountService;
import com.hfkj.sysenum.user.UserAccountRecordSourceTypeEnum;
import com.hfkj.sysenum.user.UserAccountRecordTypeEnum;
import com.hfkj.sysenum.user.UserAccountStatusEnum;
import org.apache.commons.collections4.MapUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @className: BsUserAccountServiceImpl
 * @author: HuRui
 * @date: 2024/9/8
 **/
@Service("userAccountService")
public class BsUserAccountServiceImpl implements BsUserAccountService {
    @Resource
    private BsUserAccountMapper userAccountMapper;
    @Resource
    private BsUserAccountRecordService userAccountRecordService;
    @Resource
    private RedisTemplate<String,Object> redisTemplate;
    private final String LOCK_KEY = "USER_TRADE_LOCK_";

    @Override
    public void create(Long userId) {
        BsUserAccount account = new BsUserAccount();
        account.setUserId(userId);
        account.setGoldCoin(new BigDecimal("0"));
        account.setStatus(UserAccountStatusEnum.status1.getCode());
        account.setCreateTime(new Date());
        account.setUpdateTime(new Date());
        userAccountMapper.insert(account);
    }

    @Override
    public BsUserAccount getAccount(Long userId) {
        BsUserAccountExample example = new BsUserAccountExample();
        example.createCriteria().andUserIdEqualTo(userId).andStatusNotEqualTo(UserAccountStatusEnum.status0.getCode());
        List<BsUserAccount> list = userAccountMapper.selectByExample(example);
        if (!list.isEmpty()) {
            return list.get(0);
        }
        return null;
    }

    /**
     * 修改数据
     * @param data
     */
    public void update(BsUserAccount data) {
        data.setUpdateTime(new Date());
        userAccountMapper.updateByPrimaryKey(data);
    }

    @Override
    @Transactional(propagation= Propagation.REQUIRED,rollbackFor= {RuntimeException.class})
    public void recharge(Long userId, BigDecimal amount, UserAccountRecordSourceTypeEnum sourceType, Map<String, Object> otherParam) throws Exception {
        // 锁编号
        String lockKey = LOCK_KEY+userId;
        // 获取锁
        Boolean lock = redisTemplate.opsForValue().setIfAbsent(lockKey, "");
        if (Boolean.TRUE.equals(lock)) {
            try {
                // 获取锁成功
                // 锁超时时间 10秒
                redisTemplate.expire(lockKey, 10, TimeUnit.SECONDS);

                // 查询账户
                BsUserAccount userAccount = getAccount(userId);
                // 变更前金额
                BigDecimal beforeAmount = userAccount.getGoldCoin();
                // 计算金额
                userAccount.setGoldCoin(userAccount.getGoldCoin().add(amount));
                update(userAccount);
                // 变更后金额
                BigDecimal afterAmount = userAccount.getGoldCoin();

                BsUserAccountRecord record = new BsUserAccountRecord();
                record.setUserId(userId);
                record.setUserAccountId(userAccount.getId());
                record.setType(UserAccountRecordTypeEnum.type1.getType());
                record.setTransactionAmount(amount);
                record.setBeforeAmount(beforeAmount);
                record.setAfterAmount(afterAmount);
                record.setSourceType(sourceType.getType());
                record.setSourceId(MapUtils.getLong(otherParam, "sourceId"));
                record.setSourceOrderNo(MapUtils.getString(otherParam, "sourceOrderNo"));
                record.setSourceContent(MapUtils.getString(otherParam, "sourceContent"));
                userAccountRecordService.create(record);

                // 释放锁
                redisTemplate.delete(lockKey);

            } catch (BaseException e) {
                // 释放锁
                redisTemplate.delete(lockKey);
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, e.getErrorMsg());
            } catch (Exception e) {
                // 释放锁
                redisTemplate.delete(lockKey);
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "啊偶~交易出现未知问题！请稍后重试");
            }
        } else {
            Thread.sleep(100);
            recharge(userId, amount, sourceType, otherParam);
        }
    }

    @Override
    public void consume(Long userId, BigDecimal amount, UserAccountRecordSourceTypeEnum sourceType, Map<String, Object> otherParam) throws Exception {
        // 锁编号
        String lockKey = LOCK_KEY+userId;
        // 获取锁
        Boolean lock = redisTemplate.opsForValue().setIfAbsent(lockKey, "");
        if (Boolean.TRUE.equals(lock)) {
            try {
                // 获取锁成功
                // 锁超时时间 10秒
                redisTemplate.expire(lockKey, 10, TimeUnit.SECONDS);

                // 查询账户
                BsUserAccount userAccount = getAccount(userId);
                // 变更前金额
                BigDecimal beforeAmount = userAccount.getGoldCoin();
                // 计算金额
                userAccount.setGoldCoin(userAccount.getGoldCoin().subtract(amount));
                update(userAccount);
                // 变更后金额
                BigDecimal afterAmount = userAccount.getGoldCoin();

                BsUserAccountRecord record = new BsUserAccountRecord();
                record.setUserId(userId);
                record.setUserAccountId(userAccount.getId());
                record.setType(UserAccountRecordTypeEnum.type1.getType());
                record.setTransactionAmount(amount);
                record.setBeforeAmount(beforeAmount);
                record.setAfterAmount(afterAmount);
                record.setSourceType(sourceType.getType());
                record.setSourceId(MapUtils.getLong(otherParam, "sourceId"));
                record.setSourceOrderNo(MapUtils.getString(otherParam, "sourceOrderNo"));
                record.setSourceContent(MapUtils.getString(otherParam, "sourceContent"));
                userAccountRecordService.create(record);

                // 释放锁
                redisTemplate.delete(lockKey);

            } catch (BaseException e) {
                // 释放锁
                redisTemplate.delete(lockKey);
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, e.getErrorMsg());
            } catch (Exception e) {
                // 释放锁
                redisTemplate.delete(lockKey);
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "啊偶~交易出现未知问题！请稍后重试");
            }
        } else {
            Thread.sleep(100);
            consume(userId, amount, sourceType, otherParam);
        }
    }
}
