package com.hfkj.service.user.impl;

import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.security.AESEncodeUtil;
import com.hfkj.common.security.SessionObject;
import com.hfkj.common.security.UserCenter;
import com.hfkj.common.utils.RedisUtil;
import com.hfkj.dao.BsUserMapper;
import com.hfkj.entity.BsUser;
import com.hfkj.entity.BsUserExample;
import com.hfkj.entity.BsUserInviteCode;
import com.hfkj.model.UserSessionObject;
import com.hfkj.service.user.BsUserAccountService;
import com.hfkj.service.user.BsUserInviteCodeService;
import com.hfkj.service.user.BsUserLoginLogService;
import com.hfkj.service.user.BsUserService;
import com.hfkj.sysenum.user.UserGradeEnum;
import com.hfkj.sysenum.user.UserLoginType;
import com.hfkj.sysenum.user.UserStatusEnum;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @className: BsUserServiceImpl
 * @author: HuRui
 * @date: 2024/9/6
 **/
@Service("userService")
public class BsUserServiceImpl implements BsUserService {
    @Resource
    private BsUserMapper userMapper;
    @Resource
    private RedisUtil redisUtil;
    @Autowired
    private UserCenter userCenter;
    @Resource
    private BsUserLoginLogService userLoginLogService;
    @Resource
    private BsUserAccountService userAccountService;
    @Resource
    private BsUserInviteCodeService userInviteCodeService;
    private final static String CACHE_KEY = "USER";

    /**
     * 用户token
     * @param userId
     * @return
     */
    public static String userToken(Long userId) throws Exception {
        return AESEncodeUtil.aesEncrypt(userId.toString());
    }

    @Override
    public void editData(BsUser data) {
        data.setUpdateTime(new Date());
        if (data.getId() == null) {
            data.setCreateTime(new Date());
            userMapper.insert(data);
        } else {
            userMapper.updateByPrimaryKey(data);
        }
        if (StringUtils.isNotBlank(data.getPhone())) {
            // 加入缓存
            redisUtil.hset(CACHE_KEY, data.getPhone(), data);
        }
    }

    @Override
    public void updatePhone(Long userId, String newPhone) throws Exception {
        if (getUser(newPhone) != null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "");
        }
        // 查询用户
        BsUser user = getUser(userId);
        if (user == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未找到用户");
        }
        user.setPhone(newPhone);
        editData(user);

        // 更新session
        UserSessionObject session = new UserSessionObject();
        session.setUser(user);
        SessionObject sessionObject = new SessionObject(userToken(user.getId()), session);
        userCenter.save(sessionObject);
    }

    @Override
    public void updateWechatNum(Long userId, String wechatNum) throws Exception {
        // 查询用户
        BsUser user = getUser(userId);
        if (user == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未找到用户");
        }
        user.setWechatNum(wechatNum);
        editData(user);

        // 更新session
        UserSessionObject session = new UserSessionObject();
        session.setUser(user);
        SessionObject sessionObject = new SessionObject(userToken(user.getId()), session);
        userCenter.save(sessionObject);
    }

    @Override
    public BsUser getUser(Long userId) {
        return userMapper.selectByPrimaryKey(userId);
    }

    @Override
    public BsUser getUser(String userPhone) {
        // 查询缓存
        Object cacheObj = redisUtil.hget(CACHE_KEY, userPhone);
        if (cacheObj != null) {
            return (BsUser) cacheObj;
        }
        BsUserExample example = new BsUserExample();
        example.createCriteria().andPhoneEqualTo(userPhone).andStatusNotEqualTo(UserStatusEnum.status0.getCode());
        List<BsUser> list = userMapper.selectByExample(example);
        if (!list.isEmpty()) {
            BsUser user = list.get(0);
            // 加入缓存
            redisUtil.hset(CACHE_KEY, user.getPhone(), user);
            return user;
        }
        return null;
    }

    @Override
    @Transactional(propagation= Propagation.REQUIRED,rollbackFor= {RuntimeException.class})
    public BsUser register(String phone, Map<String, Object> other, String inviteCode) {
        // 查询手机号
        if (getUser(phone) != null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "用户手机号已存在，请更换");
        }
        if (StringUtils.isNotBlank(inviteCode)) {
            // 校验邀请码
            BsUserInviteCode userInviteCode = userInviteCodeService.getDetail(inviteCode);
            if (userInviteCode == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "无效的邀请码");
            }
        }

        BsUser user = new BsUser();
        user.setHeadImg(MapUtils.getString(other,"headImg"));
        user.setName(
                StringUtils.isNotBlank(MapUtils.getString(other,"name"))
                ? MapUtils.getString(other,"name")
                : "元气优淘"
        );
        user.setPhone(phone);
        user.setGrade(UserGradeEnum.grade1.getCode());
        user.setStatus(UserStatusEnum.status1.getCode());
        editData(user);

        // 创建账户
        userAccountService.create(user.getId());
        return user;
    }


    @Override
    @Transactional(propagation= Propagation.REQUIRED,rollbackFor= {RuntimeException.class})
    public SessionObject login(String phone, UserLoginType loginType, Map<String, Object> other, String inviteCode) throws Exception {
        // 查询用户
        BsUser user = getUser(phone);
        if (user == null) {
            user = register(phone, other, inviteCode);
        }
        // 缓存
        UserSessionObject session = new UserSessionObject();
        session.setUser(user);

        SessionObject sessionObject = new SessionObject(userToken(user.getId()), session);
        userCenter.save(sessionObject);

        // 异步记录登录信息
        userLoginLogService.asyncCreateLog(loginType, user, (HttpServletRequest) RequestContextHolder.getRequestAttributes().resolveReference(RequestAttributes.REFERENCE_REQUEST));
        return sessionObject;
    }
}
